import { extend, sorted } from './utils/helper.esm.js';
import MatchDate from './matcher/date/matching.esm.js';
import MatchDictionary from './matcher/dictionary/matching.esm.js';
import MatchRegex from './matcher/regex/matching.esm.js';
import MatchRepeat from './matcher/repeat/matching.esm.js';
import MatchSequence from './matcher/sequence/matching.esm.js';
import MatchSpatial from './matcher/spatial/matching.esm.js';
import MatchSeparator from './matcher/separator/matching.esm.js';

class Matching {
  constructor(options) {
    this.options = options;
    this.matchers = {
      date: MatchDate,
      dictionary: MatchDictionary,
      regex: MatchRegex,
      // @ts-ignore => TODO resolve this type issue. This is because it is possible to be async
      repeat: MatchRepeat,
      sequence: MatchSequence,
      spatial: MatchSpatial,
      separator: MatchSeparator
    };
  }
  matcherFactory(name) {
    if (!this.matchers[name] && !this.options.matchers[name]) {
      return null;
    }
    const Matcher = this.matchers[name] ? this.matchers[name] : this.options.matchers[name].Matching;
    return new Matcher(this.options);
  }
  processResult(matches, promises, result) {
    if (result instanceof Promise) {
      result.then(response => {
        extend(matches, response);
      });
      promises.push(result);
    } else {
      extend(matches, result);
    }
  }
  handlePromises(matches, promises) {
    if (promises.length > 0) {
      return new Promise((resolve, reject) => {
        Promise.all(promises).then(() => {
          resolve(sorted(matches));
        }).catch(error => {
          reject(error);
        });
      });
    }
    return sorted(matches);
  }
  match(password, userInputsOptions) {
    const matches = [];
    const promises = [];
    const matchers = [...Object.keys(this.matchers), ...Object.keys(this.options.matchers)];
    matchers.forEach(key => {
      const matcher = this.matcherFactory(key);
      if (!matcher) {
        return;
      }
      const result = matcher.match({
        password,
        omniMatch: this,
        userInputsOptions
      });
      // extends matches and promises by references
      this.processResult(matches, promises, result);
    });
    return this.handlePromises(matches, promises);
  }
}

export { Matching as default };
//# sourceMappingURL=Matching.esm.js.map
