'use strict';

var helper = require('./utils/helper.js');
var matching = require('./matcher/date/matching.js');
var matching$1 = require('./matcher/dictionary/matching.js');
var matching$2 = require('./matcher/regex/matching.js');
var matching$3 = require('./matcher/repeat/matching.js');
var matching$4 = require('./matcher/sequence/matching.js');
var matching$5 = require('./matcher/spatial/matching.js');
var matching$6 = require('./matcher/separator/matching.js');

class Matching {
  constructor(options) {
    this.options = options;
    this.matchers = {
      date: matching,
      dictionary: matching$1,
      regex: matching$2,
      // @ts-ignore => TODO resolve this type issue. This is because it is possible to be async
      repeat: matching$3,
      sequence: matching$4,
      spatial: matching$5,
      separator: matching$6
    };
  }
  matcherFactory(name) {
    if (!this.matchers[name] && !this.options.matchers[name]) {
      return null;
    }
    const Matcher = this.matchers[name] ? this.matchers[name] : this.options.matchers[name].Matching;
    return new Matcher(this.options);
  }
  processResult(matches, promises, result) {
    if (result instanceof Promise) {
      result.then(response => {
        helper.extend(matches, response);
      });
      promises.push(result);
    } else {
      helper.extend(matches, result);
    }
  }
  handlePromises(matches, promises) {
    if (promises.length > 0) {
      return new Promise((resolve, reject) => {
        Promise.all(promises).then(() => {
          resolve(helper.sorted(matches));
        }).catch(error => {
          reject(error);
        });
      });
    }
    return helper.sorted(matches);
  }
  match(password, userInputsOptions) {
    const matches = [];
    const promises = [];
    const matchers = [...Object.keys(this.matchers), ...Object.keys(this.options.matchers)];
    matchers.forEach(key => {
      const matcher = this.matcherFactory(key);
      if (!matcher) {
        return;
      }
      const result = matcher.match({
        password,
        omniMatch: this,
        userInputsOptions
      });
      // extends matches and promises by references
      this.processResult(matches, promises, result);
    });
    return this.handlePromises(matches, promises);
  }
}

module.exports = Matching;
//# sourceMappingURL=Matching.js.map
